package handler

import (
	"context"
	"fmt"
	"net/http"
	"os"
	"os/signal"
	"time"

	"github.com/labstack/echo/v4"
	echoSwagger "github.com/swaggo/echo-swagger"
	_ "gitlab.informatika.org/k-01-11/graduit-be/docs"
	"gitlab.informatika.org/k-01-11/graduit-be/src/middleware"
)

// @title API DOC GRADUIT BE S1
// @version 1.0
// @description This is the API Documentation for Graduit-BE S1
// @termsOfService http://swagger.io/terms/

// @host localhost:8080
// @BasePath /
// @schemes http
func (s *Service) InitializeRoutes() *echo.Echo {
	e := echo.New()
	e.GET("/swagger/*", echoSwagger.WrapHandler)
	e.Use(middleware.CORSWithConfig(middleware.CORSConfig{
		AllowOrigins:     []string{"http://localhost:5173", "https://www.domainbuattesting.my.id"},
		AllowMethods:     []string{echo.GET, echo.POST, echo.PUT, echo.DELETE, echo.PATCH},
		AllowHeaders:     []string{echo.HeaderOrigin, echo.HeaderContentType, echo.HeaderAccept},
		AllowCredentials: true,
	}))
	internalGroup := e.Group("api/internal")
	internalGroup.Use(middleware.BasicAuth())
	s.InternalSystemLogHandler.MountInternal(internalGroup)

	adminGroup := e.Group("api/admin")
	adminGroup.Use(middleware.Validator("TIMTA"))
	s.AdminSytemLogHandler.MountAdmin(adminGroup)
	s.AdminBimbinganLogHandler.MountAdmin(adminGroup)
	s.AdminTopicAllocationHandler.MountAdmin(adminGroup)
	s.BatchSeminarHandler.MountAdmin(adminGroup)
	s.BatchSidangHandler.MountAdmin(adminGroup)
	s.AdminPendaftaranSidSemHandler.MountAdmin(adminGroup)
	s.AdminPendaftaranHandler.MountAdmin(adminGroup)
	s.AdminPeriodePendaftaranHandler.MountAdmin(adminGroup)
	s.AdminDashboardTimTugasHandler.MountAdmin(adminGroup)
	s.AdminRuanganHandler.MountAdmin(adminGroup)

	mahasiswaGroup := e.Group("api/mahasiswa")
	mahasiswaGroup.Use(middleware.Validator("S1_MAHASISWA"))
	s.MahasiswaBimbinganLogHandler.MountMahasiswa(mahasiswaGroup)
	s.MahasiswaPendaftaranSidSemHandler.MountMahasiswa(mahasiswaGroup)
	s.MahasiswaDashboardHandler.MountMahasiswa(mahasiswaGroup)

	dosbingGroup := e.Group("api/dosbing")
	dosbingGroup.Use(middleware.Validator("S1_PEMBIMBING"))
	s.DosbingBimbinganLogHandler.MountDosbing(dosbingGroup)
	s.DosbingDashboardHandler.MountDosbing(dosbingGroup)

	TIMTAGroup := e.Group("api/TIMTA")
	TIMTAGroup.Use(middleware.Validator("S1_TIM_TA"))
	s.TIMTASchedulingHandler.MountTIMTA(TIMTAGroup)
	s.TIMTAPendaftaranSidSemHandler.MountTIMTA(TIMTAGroup)
	return e

}

func (s *Service) StartServer() {
	server := s.InitializeRoutes()

	listenerPort := fmt.Sprintf(":%v", 8080)
	if err := server.StartServer(&http.Server{
		Addr:         listenerPort,
		ReadTimeout:  120 * time.Second,
		WriteTimeout: 120 * time.Second,
	}); err != nil {
		server.Logger.Fatal(err.Error())
	}
}

func (s *Service) ShutdownServer() {
	server := s.InitializeRoutes()

	quit := make(chan os.Signal, 1)
	signal.Notify(quit, os.Interrupt)
	<-quit

	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
	defer cancel()

	if err := server.Shutdown(ctx); err != nil {
		server.Logger.Fatal(err.Error())
	}
}
