package transport

import (
	"net/http"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_sidang/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_sidang/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_sidang/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_sidang/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type AdminBatchSidangHandler struct {
	batchSidangUsecase usecase.BatchSidangUsecase
}

func NewAdminBatchSidangHandler(cfg config.BatchSidangTransportConfig) *AdminBatchSidangHandler {
	batchSidangRepository := repository.NewBatchSidangRepository(cfg.DBWrite, cfg.DBRead)
	batchSidangUsecase := usecase.NewBatchSidangUseCase(batchSidangRepository)

	return &AdminBatchSidangHandler{
		batchSidangUsecase: batchSidangUsecase,
	}

}

func (t *AdminBatchSidangHandler) MountAdmin(group *echo.Group) {
	group.POST("/batch-sidang", t.AddBatchSidang)
	group.GET("/batch-sidang", t.GetBatchSidang)
	group.DELETE("/batch-sidang", t.DeleteBatch)
	group.GET("batch-sidang-current", t.GetBatchSidangCurrent)
	group.PATCH("/batch-sidang-live", t.SetLive)
	group.GET("/batch-sidang-live", t.GetLive)
}

// AddBatchSidang adds a new batch for Sidang.
// @Summary Add batch for Sidang
// @Description Adds a new batch for Sidang.
// @Tags Batch Sidang
// @Accept json
// @Produce json
// @Param batch body entity.BatchSidang true "Batch Sidang object"
// @Success 200 {object} entity.BatchSidang "Batch added successfully"
// @Router /batch-sidang [post]
func (t *AdminBatchSidangHandler) AddBatchSidang(c echo.Context) error {
	request := new(entity.BatchSidang)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	target := &entity.BatchSidang{
		StartDate: request.StartDate,
		EndDate:   request.EndDate,
	}
	newBatch, err := t.batchSidangUsecase.AddBatchSidang(*target)
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusCreated, "Batch added successfully", newBatch))
}

// GetBatchSidang retrieves a list of batch for Sidang.
// @Summary Get batch for Sidang
// @Description Retrieves a list of batch for Sidang.
// @Tags Batch Sidang
// @Accept json
// @Produce json
// @Success 200 {object} []entity.BatchSidang "Batch Sidang successfully retreived"
// @Router /batch-sidang [get]
func (t *AdminBatchSidangHandler) GetBatchSidang(c echo.Context) error {
	batch, err := t.batchSidangUsecase.GetBatchSidang()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Batch Sidang successfully retreived", batch))
}

// DeleteBatchSidang deletes a batch for Sidang.
// @Summary Delete batch for Sidang
// @Description Deletes a batch for Sidang.
// @Tags Batch Sidang
// @Accept json
// @Produce json
// @Param batch body entity.BatchSidang true "Batch Sidang object"
// @Success 200 {object} entity.BatchSidang "Batch TA successfully deleted"
// @Router /batch-sidang [delete]
func (t *AdminBatchSidangHandler) DeleteBatch(c echo.Context) error {
	request := new(entity.BatchSidang)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}
	batch, err := t.batchSidangUsecase.DeleteBatchSidang(request.IdBatch)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Batch sidang successfully deleted", batch))
}

// GetBatchSidangCurrent retrieves a list of batch for Sidang that current time is between its startDate and endDate.
// @Summary Get batch sidang that current time is between its startDate and endDate.
// @Description Retrieves a list of batch for Sidang.
// @Tags Batch Sidang
// @Accept json
// @Produce json
// @Success 200 {object} []entity.BatchSidang "Batch sidang successfully retreived"
// @Router /batch-sidang-current [get]
func (t *AdminBatchSidangHandler) GetBatchSidangCurrent(c echo.Context) error {
	batch, err := t.batchSidangUsecase.GetBatchSidangCurrent()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Batch sidang successfully retreived", batch))
}

// SetLive sets a batch for sidang to be live.
// @Summary Set Batch to live
// @Description Set Batch to live.
// @Tags Batch Sidang
// @Accept json
// @Produce json
// @Param batch body entity.BatchSeminar true "Batch Sidang object"
// @Success 200 {object} entity.BatchSeminar  "Batch sidang successfully updated to live"
// @Router /batch-sidang-live [patch]
func (t *AdminBatchSidangHandler) SetLive(c echo.Context) error {
	request := new(entity.BatchSidang)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	batch, err := t.batchSidangUsecase.SetLiveBatch(request.IdBatch)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Batch sidang successfully updated to live", batch))
}

// GetLive retrieves a batch for sidang that its status is live.
// @Summary Get batch sidang that is live.
// @Description Retrieves a batch for sidang.
// @Tags Batch Sidang
// @Accept json
// @Produce json
// @Success 200 {object} []entity.BatchSidang "Batch seminar successfully retreived"
// @Router /batch-sidang-live [get]
func (t *AdminBatchSidangHandler) GetLive(c echo.Context) error {
	batch, err := t.batchSidangUsecase.GetBatchSidangLive()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Batch sidang successfully retreived", batch))
}
