package transport

import (
	"net/http"
	"strconv"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_dosbing/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_dosbing/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_dosbing/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type DosbingDashboardHandler struct {
	dashboardDosbingUseCase usecase.DashboardDosbingUseCase
}

func NewDosbingDashboardHandler(cfg config.DashboardDosbingTransportConfig) *DosbingDashboardHandler {
	dashboardDosbingRepository := repository.NewDashboardDosbingRepository(cfg.DBWrite, cfg.DBRead)
	dashboardDosbingUsecase := usecase.NewDashboardDosbingUsecase(dashboardDosbingRepository)

	return &DosbingDashboardHandler{
		dashboardDosbingUseCase: dashboardDosbingUsecase,
	}
}

func (t *DosbingDashboardHandler) MountDosbing(group *echo.Group) {
	group.GET("/dashboard", t.GetMahasiswa)
	group.GET("/dashboard/statistic", t.GetAmountMahasiswa)
	group.GET("/dashboard/status-bimbingan", t.GetAmountStatusBimbinganMahasiswa)
}

// GetMahasiswa retrieves a list of mahasiswa supervised by a specific dosen for TA.
// @Summary Get mahasiswa by dosen ID
// @Description Retrieves a list of mahasiswa supervised by a specific dosen for TA.
// @Tags Dashboard Dosbing
// @Accept json
// @Produce json
// @Param search query string false "Search"
// @Param size query string false "Size"
// @Param offset query string false "Offset"
// @Success 200 {object} []entity.MahasiswaBimbingan "List mahasiswa successfully retrieved"
// @Router /api/dosbing/dashboard [get]
func (t *DosbingDashboardHandler) GetMahasiswa(c echo.Context) error {
	payload, ok := c.Get("authPayload").(utils.Payload)

	if !ok {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, "Error parsing id", nil))
	}

	idDosen := payload.ID
	sizeStr := c.QueryParam("size")
	offsetStr := c.QueryParam("offset")
	search := c.QueryParam("search")

	if idDosen == "" {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, "insert id_dosen", nil))
	}

	size, err := strconv.Atoi(sizeStr)
	if err != nil {
		// Handle error, maybe return an error response
		size = 10
	}

	offset, err := strconv.Atoi(offsetStr)
	if err != nil {
		// Handle error, maybe return an error response
		offset = 0
	}

	listMahasiswa, err := t.dashboardDosbingUseCase.GetMahasiswaByDosenId(idDosen, offset, size, search)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "List mahasiswa successfully retreived", listMahasiswa))

}

// GetAmountMahasiswa retrieves the amount of mahasiswa supervised by a specific dosen for TA.
// @Summary Get amount of mahasiswa
// @Description Retrieves the amount of mahasiswa supervised by a specific dosen for TA.
// @Tags Dashboard Dosbing
// @Accept json
// @Produce json
// @Success 200 {object} entity.JumlahMahasiswa "List mahasiswa successfully retreived"
// @Router /api/dosbing/dashboard/statistic [get]
func (t *DosbingDashboardHandler) GetAmountMahasiswa(c echo.Context) error {
	payload, ok := c.Get("authPayload").(utils.Payload)

	if !ok {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, "Error parsing id", nil))
	}

	idDosen := payload.ID

	if idDosen == "" {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, "insert id_dosen", nil))
	}

	listMahasiswa, err := t.dashboardDosbingUseCase.GetAmountMahasiswaByDosenId(idDosen)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "List mahasiswa successfully retreived", listMahasiswa[0]))

}

// GetAmountStatusBimbinganMahasiswa retrieves the amount of mahasiswa's bimbingan status supervised by a specific dosen for TA.
// @Summary Get amount of mahasiswa's bimbingan status by dosen ID
// @Description Retrieves the amount of mahasiswa's bimbingan status supervised by a specific dosen for TA.
// @Tags Dashboard Dosbing
// @Accept json
// @Produce json
// @Success 200 {object} entity.StatusBimbinganMahasiswa "List mahasiswa successfully retreived"
// @Router /api/dosbing/dashboard/status-bimbingan [get]
func (t *DosbingDashboardHandler) GetAmountStatusBimbinganMahasiswa(c echo.Context) error {
	payload, ok := c.Get("authPayload").(utils.Payload)

	if !ok {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, "Error parsing id", nil))
	}

	idDosen := payload.ID

	if idDosen == "" {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, "insert id_dosen", nil))
	}

	listMahasiswa, err := t.dashboardDosbingUseCase.GetAmountStatusBimbinganMahasiswaByDosenId(idDosen)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "List mahasiswa successfully retreived", listMahasiswa[0]))

}
